/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlException;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.SoapuiWorkspaceDocumentConfig;
import com.eviware.soapui.impl.actions.ImportWsdlProjectAction;
import com.eviware.soapui.impl.actions.NewWsdlProjectAction;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.support.AbstractModelItem;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.tree.nodes.WorkspaceTreeNode;
import com.eviware.soapui.model.workspace.Workspace;
import com.eviware.soapui.model.workspace.WorkspaceListener;
import com.eviware.soapui.support.ExtensionFileFilter;

/**
 * Default Workspace implementation
 * 
 * @author Ole.Matzura
 */

public class WorkspaceImpl extends AbstractModelItem implements Workspace
{
   private final static Logger log = Logger.getLogger( WorkspaceImpl.class );
   
   private List<Project> projectList = new ArrayList<Project>();

   private SoapuiWorkspaceDocumentConfig workspaceConfig;

   private String path = null;

   private Set<WorkspaceListener> listeners = new HashSet<WorkspaceListener>();

   private JFileChooser chooser;

   private ImageIcon workspaceIcon;

   public WorkspaceImpl(String path) throws XmlException, IOException
   {
      this.path = path;

      File file = new File(path);
      if (file.exists())
      {
         log.info( "Loading workspace from [" + file.getAbsolutePath() + "]" );
         workspaceConfig = SoapuiWorkspaceDocumentConfig.Factory.parse(file);

         String[] projects = workspaceConfig.getSoapuiWorkspace().getProjectArray();
         for (int i = 0; i < projects.length; i++)
         {
            String str = projects[i];
            if( new File( str ).exists() )
            {
               try
               {
                  WsdlProject project = new WsdlProject(str);
                  projectList.add(project);
               }
               catch( Exception e )
               {
                  JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), 
                  		"Failed to load project [" + str + "]\nfrom workspace; " + e.getMessage(), 
                  		"Load Workspace", JOptionPane.ERROR_MESSAGE );
               }
            }
            else
            {
               JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), "project [" + str + 
                     "]\nnot found, removing from workspace",
                     "Load Workspace", JOptionPane.ERROR_MESSAGE );
            }
         }
      }
      else
      {
         workspaceConfig = SoapuiWorkspaceDocumentConfig.Factory.newInstance();
         workspaceConfig.addNewSoapuiWorkspace().setName("Default Workspace");
      }

      initActions();
      
      workspaceIcon = SoapUI.createImageIcon("/workspace.gif");
   }

   public JFileChooser getChooser()
   {
      if( chooser == null )
      {
         chooser = new JFileChooser();
         chooser.setDialogTitle( "Create New Project");
         chooser.setAcceptAllFileFilterUsed( true );
         chooser.setFileFilter( new ExtensionFileFilter( ".xml", "XML Files (*.xml)"));
      }
      return chooser;
   }
   
   public void setPath(String path)
   {
      this.path = path;
   }

   public Project[] getProjects()
   {
      return projectList.toArray(new Project[projectList.size()]);
   }

   public void setName(String name)
   {
      workspaceConfig.getSoapuiWorkspace().setName(name);
   }

   public String getName()
   {
      return "Projects"; //workspaceConfig.getSoapuiWorkspace().getName();
   }

   public Project getProjectAt(int index)
   {
      return projectList.get(index);
   }

   public int getProjectCount()
   {
      return projectList.size();
   }

   public void onClose()
   {
      save();
   }

   public void save()
   {
      try
		{
			List<String> paths = new ArrayList<String>();

			// save projects first
			for (int c = 0; c < getProjectCount(); c++)
			{
				WsdlProject project = (WsdlProject) getProjectAt(c);
				String path = project.save();
				if (path != null)
					paths.add(path);
			}

			if (path == null)
			{
				JFileChooser chooser = getChooser();
				chooser.setDialogTitle("Save workspace");
				chooser.setAcceptAllFileFilterUsed( true );
	         chooser.setFileFilter( new ExtensionFileFilter( ".xml", "XML Files (*.xml)"));
				if (chooser.showSaveDialog(SoapUI.getInstance().getFrame()) != JFileChooser.APPROVE_OPTION)
					return;

				path = chooser.getSelectedFile().getAbsolutePath();
			}

			workspaceConfig.getSoapuiWorkspace().setProjectArray(
					paths.toArray(new String[paths.size()]));
			File workspaceFile = new File(path);
			workspaceConfig.save(workspaceFile);
			
			log.info( "Saved workspace to [" + workspaceFile.getAbsolutePath() + "]" );
		}
		catch (Exception e)
		{
			log.error( "Failed to save workspace: " + e.getMessage(), e );
		}
   }

   private void initActions()
   {
      addAction( new NewWsdlProjectAction( this ) );
      addAction( new ImportWsdlProjectAction( this ) );
   }

   public void addWorkspaceListener(WorkspaceListener listener)
   {
      listeners.add(listener);
   }

   public void removeWorkspaceListener(WorkspaceListener listener)
   {
      listeners.remove(listener);
   }

   public void importProject(String fileName)
   {
      File projectFile = new File( fileName );
      
      try
      {
         WsdlProject project = new WsdlProject( projectFile.getAbsolutePath() );
         projectList.add(project);

         for (Iterator<WorkspaceListener> iter = listeners.iterator(); iter.hasNext();)
         {
            WorkspaceListener listener = iter.next();
            listener.projectAdded(project);
         }
         
         save();
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }
   
   public void createProject(String name)
   {
      JFileChooser chooser = getChooser();
      chooser.setDialogTitle("Create Project");
      chooser.setAcceptAllFileFilterUsed( true );
      chooser.setFileFilter( new ExtensionFileFilter( ".xml", "XML Files (*.xml)"));
      File projectFile = new File( createProjectFileName( name ));
      
      chooser.setSelectedFile( projectFile);
      
      if (chooser.showSaveDialog(SoapUI.getInstance().getFrame()) != JFileChooser.APPROVE_OPTION)
         return;

      projectFile = chooser.getSelectedFile();
      while( projectFile.exists())
      {
         int result = JOptionPane.NO_OPTION;
         while( result == JOptionPane.NO_OPTION )
         { 
            result = JOptionPane.showConfirmDialog(SoapUI.getInstance().getFrame(), "File exists, overwrite?", 
                  "Overwrite project?", JOptionPane.YES_NO_CANCEL_OPTION );
            if( result == JOptionPane.CANCEL_OPTION ) return;
            if( result == JOptionPane.YES_OPTION )
            {
               projectFile.delete();
            }
            else
            {
               if (chooser.showSaveDialog(SoapUI.getInstance().getFrame()) != JFileChooser.APPROVE_OPTION)
                  return;
               
               projectFile = chooser.getSelectedFile();
            }
         }
      }

      try
      {
         WsdlProject project = new WsdlProject( projectFile.getAbsolutePath() );
         
         project.setName(name);
         projectList.add(project);

         for (Iterator<WorkspaceListener> iter = listeners.iterator(); iter.hasNext();)
         {
            WorkspaceListener listener = iter.next();
            listener.projectAdded(project);
         }
         
         save();
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }

   private String createProjectFileName(String name)
   {
      StringBuffer result = new StringBuffer();
      for( int c = 0; c < name.length(); c++ )
      {
         char ch = name.charAt( c );
         if( Character.isLetterOrDigit( ch ))
         {
            result.append( ch );
         }
      }
      
      result.append( "-soapui-project.xml" );
      return result.toString();
   }

  

   public void removeProject(Project project)
   {
		int ix = projectList.indexOf( project );
		if( ix == -1 )
			throw new RuntimeException( "Project [" + project.getName() + "] not available in workspace for removal" );
		
		WorkspaceListener[] listenerArray = listeners.toArray( new WorkspaceListener[listeners.size()] );
		for( int c = 0; c < listenerArray.length; c++ )
      {
         listenerArray[c].projectRemoved(project);
      }
      
      projectList.remove( ix );
      workspaceConfig.getSoapuiWorkspace().removeProject( ix );
   }

   public ImageIcon getIcon()
   {
      return workspaceIcon;
   }

   protected SoapUITreeNode createTreeNode()
   {
      return new WorkspaceTreeNode( this );
   }
}
