/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.CallConfig;
import com.eviware.soapui.config.OperationConfig;
import com.eviware.soapui.impl.wsdl.actions.operation.NewRequestAction;
import com.eviware.soapui.impl.wsdl.actions.operation.RelabelOperationAction;
import com.eviware.soapui.impl.wsdl.panels.operation.WsdlOperationPanelBuilder;
import com.eviware.soapui.impl.wsdl.support.SoapRequestBuilder;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.tree.nodes.OperationTreeNode;

/**
 * WSDL implementation of Operation, maps to a WSDL BindingOperation
 * 
 * @author Ole.Matzura
 */

public class WsdlOperation extends AbstractWsdlModelItem implements Operation
{
   private OperationConfig operationConfig;
   private List<WsdlRequest> requests = new ArrayList<WsdlRequest>();
   private WsdlInterface iface;
   private ImageIcon operationIcon;
	private PanelBuilder wsdlOperationPanelBuilder;

   public WsdlOperation( WsdlInterface iface, OperationConfig operationConfig )
   {
      this.iface = iface;
      this.operationConfig = operationConfig;
      
      CallConfig[] requestConfigs = operationConfig.getCallArray();
      for (int i = 0; i < requestConfigs.length; i++)
      {
         requests.add( new WsdlRequest( this, requestConfigs[i] ));
      }
      
      addAction( new NewRequestAction( this ));
      addAction( new RelabelOperationAction( this ));
      
      operationIcon = SoapUI.createImageIcon("/operation.gif");
      wsdlOperationPanelBuilder = new WsdlOperationPanelBuilder( this );
   }
   
   public String getAction()
   {
      return operationConfig.getAction();
   }

   public Request getRequestAt(int index)
   {
      return requests.get( index );
   }

   public int getRequestCount()
   {
      return requests.size();
   }

   public String getName()
   {
      return operationConfig.getName();
   }

   public void setName(String name)
   {
      String old = getName();
      operationConfig.setName( name );
      notifyPropertyChanged( NAME_PROPERTY, old, name );
   }

   public WsdlRequest addNewRequest( String name )
   {
      WsdlRequest requestImpl = new WsdlRequest( this, operationConfig.addNewCall() );
      requestImpl.setName( name );
      requests.add( requestImpl );
      ((WsdlInterface)getInterface()).notifyRequestAdded( requestImpl );
      return requestImpl;
   }

   public Interface getInterface()
   {
      return iface;
   }

   public OperationConfig getOperationConfig()
   {
      return operationConfig;
   }

   public void setAction(String soapAction)
   {
      String old = getAction();
      operationConfig.setAction( soapAction );
      notifyPropertyChanged( ACTION_PROPERTY, old, soapAction );
   }

   public String createRequest( boolean buildOptional )
   {
      if( iface.getBindingName() == null )
      {
         JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), "Missing binding name, refresh " +
               "Interface\nfor request generation to work correctly", 
               "Create Request", JOptionPane.ERROR_MESSAGE );
         return null;
      }
      
      if( getBindingOperationName() == null )
      {
         JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), "Missing bindingOperation name, refresh " +
               "Interface\nfor request generation to work correctly",
               "Create Request", JOptionPane.ERROR_MESSAGE);
         return null;
      }
      
      try
      {
         SoapRequestBuilder builder = (SoapRequestBuilder) iface.getRequestBuilder();
         return builder.buildSoapRequest( findBindingOperation( iface.getWsdlContext().getDefinition() ), buildOptional );
      }
      catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
   public BindingOperation findBindingOperation(Definition definition)
   {
      return definition.getBinding( iface.getBindingName() ).getBindingOperation( 
            operationConfig.getBindingOperationName(), null, null );
   }
  
   public void removeRequest( WsdlRequest request )
   {
   	((WsdlInterface)getInterface()).notifyRequestRemoved( request );

   	request.release();
   	
      int ix = requests.indexOf( request );
      requests.remove( ix );
      operationConfig.removeCall( ix );
   }

   public String getBindingOperationName()
   {
      return operationConfig.getBindingOperationName();
   }
   
   public void setBindingOperationName( String name )
   {
      operationConfig.setBindingOperationName( name );
   }

   public ImageIcon getIcon()
   {
      return operationIcon;
   }

   protected SoapUITreeNode createTreeNode()
   {
      return new OperationTreeNode( this );
   }

	public PanelBuilder getPanelBuilder()
	{
		return wsdlOperationPanelBuilder;
	}
}
