 /*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionException;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.support.AbstractModelItem;
import com.eviware.soapui.model.tree.SoapUITreeNode;

/**
 * Base class for WsdlAssertions
 * 
 * @author Ole.Matzura
 */

public abstract class WsdlAssertion extends AbstractModelItem
{
   private RequestAssertionConfig assertionConfig;
   private final WsdlTestRequest request;
   private AssertionStatus assertionStatus = AssertionStatus.UNKNOWN;
   private com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError [] assertionErrors;
   private final static Log log = LogFactory.getLog( WsdlAssertion.class );
   private ImageIcon validIcon;
   private ImageIcon failedIcon;
   private ImageIcon unknownIcon;
   
   public final static String STATUS_PROPERTY = WsdlAssertion.class.getName() + "@status";  
   public final static String CONFIGURATION_PROPERTY = WsdlAssertion.class.getName() + "@configuration";  
   
   public enum AssertionStatus { UNKNOWN, VALID, FAILED, NONE };

   public WsdlAssertion(RequestAssertionConfig assertionConfig, WsdlTestRequest request)
   {
      this.assertionConfig = assertionConfig;
      this.request = request;
      
      addAction( new RenameAssertionAction() );
      addAction( new RemoveAssertionAction() );
      if( isConfigurable() )
         addAction( new ConfigureAssertionAction() );
      
      validIcon = SoapUI.createImageIcon("/valid_assertion.gif");
      failedIcon = SoapUI.createImageIcon("/failed_assertion.gif");
      unknownIcon = SoapUI.createImageIcon("/unknown_assertion.gif");      
   }
   
   protected XmlObject getConfiguration()
   {
      if( null == assertionConfig.getConfiguration())
      {
         assertionConfig.addNewConfiguration();
      }
      
      return assertionConfig.getConfiguration();
   }

   protected void setConfiguration( XmlObject configuration )
   {
	   XmlObject oldConfig = assertionConfig.getConfiguration();
      assertionConfig.setConfiguration( configuration );
      notifyPropertyChanged( WsdlAssertion.CONFIGURATION_PROPERTY, oldConfig, configuration );
   }
   
   public String getName()
   {
      return assertionConfig.isSetName() ? assertionConfig.getName() : assertionConfig.getType();
   }

   public void setName(String name)
   {
      String old = getName();
      assertionConfig.setName( name );
      notifyPropertyChanged( NAME_PROPERTY, old, name );
   }
   
   public class RenameAssertionAction extends AbstractAction
   {
      public RenameAssertionAction()
      {
         super( "Rename" );
         putValue( Action.SHORT_DESCRIPTION, "Renames this assertion" );
      }
      
      public void actionPerformed(ActionEvent e)
		{
         String name = (String) JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), 
         		"Specify name for this assertion", "Rename Assertion", JOptionPane.QUESTION_MESSAGE, null, 
         		null, WsdlAssertion.this.getName() );
         if( name == null || name.equals( WsdlAssertion.this.getName() )) return;
         
         setName( name );
      }
   }
   
   public class RemoveAssertionAction extends AbstractAction
   {
      public RemoveAssertionAction()
      {
         super( "Remove" );
         putValue( Action.SHORT_DESCRIPTION, "Removes this assertion from its request" );
      }
      
      public void actionPerformed(ActionEvent e)
		{
         if( JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog( SoapUI.getInstance().getFrame(), 
               "Remove assertion [" + WsdlAssertion.this.getName() + "] from request [" + 
               getRequest().getName() + "]", "Remove Assertion", JOptionPane.OK_CANCEL_OPTION ))
         {
            return;
         }

         request.removeAssertion( WsdlAssertion.this );
      }
   }
   
   public class ConfigureAssertionAction extends AbstractAction
   {
      public ConfigureAssertionAction()
      {
         super( "Configure" );
         putValue( Action.SHORT_DESCRIPTION, "Configures this assertion" );
      }
      
      public void actionPerformed(ActionEvent e)
		{
         configure();
      }
   }

   public AssertionStatus getStatus()
   {
      return assertionStatus;
   }

   public Request getRequest()
   {
      return request;
   }

   public com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError [] getErrors()
   {
      return assertionErrors;
   }

   public AssertionStatus assertResponse(WsdlTestRequest request)
   {
      AssertionStatus oldStatus = assertionStatus;
      ImageIcon oldIcon = getIcon();
      
      if( request.getResponseContent() == null || request.getResponseContent().trim().length() == 0 )
      {
      	assertionStatus = AssertionStatus.FAILED;
      	assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
      	                    { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError("null/empty response") };
      }
      else
      {
	      try
	      {
	      	assertRequest( request );
	         assertionStatus = AssertionStatus.VALID;
	         assertionErrors = null;
	      }
	      catch ( AssertionException e )
	      {
	      	assertionStatus = AssertionStatus.FAILED;
	      	assertionErrors = e.getErrors();
	      }
	      catch (Throwable e)
	      {
	         assertionStatus = AssertionStatus.FAILED;
	         assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
                           { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError( e.getMessage() )};
	      }
      }
      
      notifyPropertyChanged( STATUS_PROPERTY, oldStatus, assertionStatus );
      notifyPropertyChanged( ICON_PROPERTY, oldIcon, getIcon() );
      
      return assertionStatus;
   }

   protected abstract String assertRequest(WsdlTestRequest request) throws AssertionException;

   public boolean isConfigurable()
   {
      return false;
   }
   
   public void configure()
   {}

   public ImageIcon getIcon()
   {
      switch( getStatus() )
      {
         case FAILED : return failedIcon; 
         case UNKNOWN : return unknownIcon; 
         case VALID : return validIcon;
      }
      
      return null;
   }

   protected SoapUITreeNode createTreeNode()
   {
      return null;
   }

	public void updateConfig(RequestAssertionConfig config)
	{
		this.assertionConfig = config;
	}
}

  