/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.tools;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion.AssertionStatus;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestRunListener;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestSuite;

/**
 * Standalone test-runner used from maven-plugin
 * 
 * @author Ole.Matzura
 */

public class SoapUITestCaseRunner implements TestRunListener
{
	private String projectFile;
	private String testSuite;
	private String testCase;
	private List<WsdlAssertion> assertions = new ArrayList<WsdlAssertion>();
	private String endpoint;
	private final static Logger log = Logger.getLogger( SoapUITestCaseRunner.class );
	private long testCaseStartTime;

	public static void main( String [] args) throws Exception
	{
		System.out.println( "SoapUI " + SoapUI.SOAPUI_VERSION + " TestRunner");
		
		SoapUITestCaseRunner runner = new SoapUITestCaseRunner();
		runner.setProjectFile( args[0] );
		if( args.length > 1 && args[1].trim().length() > 0 )
			runner.setEndpoint( args[1] );
		
		if( args.length > 2 && args[2].trim().length() > 0 )
			runner.setTestSuite( args[2] );

		if( args.length > 3 && args[3].trim().length() > 0 )
			runner.setTestCase( args[3] );

		runner.run();
	}
	
	public void run() throws Exception
	{
		assertions.clear();
		
		WsdlProject project = new WsdlProject( projectFile );
		log.info( "Running soapui tests in project [" + project.getName() + "]" );
		
		for( int c = 0; c < project.getTestSuiteCount(); c++ )
		{
			if( testSuite == null || testSuite.equals( "*") ||
				 project.getTestSuiteAt( c ).getName().equalsIgnoreCase( testSuite ))
			{
				runSuite( project.getTestSuiteAt( c ));
			}
		}
		
		if( assertions.size() > 0 )
		{
			StringBuffer buf = new StringBuffer();
			for( int c = 0; c < assertions.size(); c++ )
			{
				WsdlAssertion assertion = assertions.get( c );
				buf.append( assertion.getName() + " in [" + assertion.getRequest().getName() + "] failed;\n" );
				buf.append( assertion.getErrors() + "\n" );
				buf.append( "Request: " + assertion.getRequest().getRequestContent() + "\n" );
				buf.append( "Response: " + ((WsdlTestRequest)assertion.getRequest()).getResponseContent() + "\n" );
			}
			throw new Exception( buf.toString() );
		}
	}

	private void runSuite(TestSuite suite)
	{
		log.info(( "Running soapui suite [" + suite.getName() + "]"));
		long start = System.currentTimeMillis();
		for( int c = 0; c < suite.getTestCaseCount(); c++ )
		{
			String name = suite.getTestCaseAt( c ).getName();
			if( testCase == null || testCase.equals("*") || 
				 name.equalsIgnoreCase( testCase ))
			{
				runTestCase( suite.getTestCaseAt( c ));
			}
			else
				System.out.println( "Skipping testcase [" + name + "], filter is [" + testCase + "]");
		}
		log.info( "soapui suite [" + suite.getName() + "] finished in " + (System.currentTimeMillis()-start) + "ms" );
	}

	private void runTestCase(TestCase testCase)
	{
		testCase.addTestRunListener( this );
		testCase.run().waitUntilFinished();
	}

	public void setProjectFile(String projectFile)
	{
		log.info( "setting projectFile to [" + projectFile + "]" );
		this.projectFile = projectFile;
	}

	public void setTestCase(String testCase)
	{
		log.info( "setting testCase to [" + testCase + "]" );
      this.testCase = testCase;
	}
	
	public void setEndpoint(String endpoint)
	{
		log.info( "setting test endpoint to [" + endpoint+ "]" );
		this.endpoint = endpoint.trim();
	}

	public void setTestSuite(String testSuite)
	{
	   log.info( "setting testSuite to [" + testSuite + "]" );
		this.testSuite = testSuite;
	}

	public boolean beforeRun(TestRunner testRunner)
	{
		testCaseStartTime = System.currentTimeMillis();
		log.info( "Running soapui testcase [" + testRunner.getTestCase().getName() + "]");
		return true;
	}

	public boolean beforeStep(TestRunner testRunner)
	{
		TestStep currentStep = testRunner.getCurrentStep();
		log.info( "runing step [" + currentStep.getName() + "]" );
		
		if( endpoint != null && endpoint.length() > 0 && currentStep instanceof WsdlTestRequestStep )
		{
			WsdlTestRequestStep requestStep = (WsdlTestRequestStep) currentStep;
			requestStep.getTestRequest().setEndpoint( endpoint );
		}
		
		return true;
	}

	public boolean afterStep(TestRunner testRunner)
	{
		TestStep currentStep = testRunner.getCurrentStep();
		
		if( currentStep instanceof WsdlTestRequestStep )
		{
			WsdlTestRequestStep requestStep = (WsdlTestRequestStep) currentStep;
			for( int c = 0; c < requestStep.getAssertionCount(); c++ )
			{
				WsdlAssertion assertion = requestStep.getAssertionAt( c );
				System.out.println( "Assertion [" + assertion.getName() + "] has status " + assertion.getStatus());
				if( assertion.getStatus() != AssertionStatus.VALID )
				{
					System.out.println( "ASSERTION FAILED -> " + assertion.getErrors());
					assertions.add( assertion );
				}
			}
		}	
		
		return true;
	}

	public void afterRun(TestRunner testRunner)
	{
		long end = System.currentTimeMillis();
		log.info( "Finished running soapui testcase [" + testRunner.getTestCase().getName() + "], time taken = " + 
				(end-testCaseStartTime) + "ms" );
	}
}